-- name: [CS] Dry Bowser
-- description: Bowser when haves no water\n\n\\#ff7777\\This Pack requires Character Select\nto use as a Library!

--[[
    API Documentation for Character Select can be found below:
    https://github.com/Squishy6094/character-select-coop/wiki/API-Documentation

    Use this if you're curious on how anything here works >v<
]]

local TEXT_MOD_NAME = "Dry Bowser"

-- Stops mod from loading if Character Select isn't on
if not _G.charSelectExists then
    djui_popup_create("\\#ffffdc\\\n"..TEXT_MOD_NAME.."\nRequires the Character Select Mod\nto use as a Library!\n\nPlease turn on the Character Select Mod\nand Restart the Room!", 6)
    return 0
end

local E_MODEL_CUSTOM_MODEL = smlua_model_util_get_id("drybowser_geo") -- Located in "actors"

local TEX_CUSTOM_LIFE_ICON = get_texture_info("drybowser_icon") -- Located in "textures"

-- All Located in "sound"
local VOICETABLE_DEDEDE = {
	--[CHAR_SOUND_OKEY_DOKEY] = 'GetStar.ogg', -- Starting game
	[CHAR_SOUND_LETS_A_GO] = 'GetStar.ogg', -- Starting level
	[CHAR_SOUND_PUNCH_YAH] = 'Punch1.ogg', -- Punch 1
	[CHAR_SOUND_PUNCH_WAH] = 'Punch2.ogg', -- Punch 2
	[CHAR_SOUND_PUNCH_HOO] = 'Punch3.ogg', -- Punch 3
	[CHAR_SOUND_YAH_WAH_HOO] = {'Jump1.ogg', 'Jump2.ogg', 'Jump3.ogg'}, -- First/Second jump sounds
	[CHAR_SOUND_HOOHOO] = 'LongJump.ogg', -- Third jump sound
	[CHAR_SOUND_YAHOO_WAHA_YIPPEE] = {'TripleJump1.ogg', 'TripleJump2.ogg'}, -- Triple jump sounds
	[CHAR_SOUND_UH] = 'Damage.ogg', -- Wall bonk
	[CHAR_SOUND_UH2] = 'Silent.ogg', -- Landing after long jump
	[CHAR_SOUND_UH2_2] = 'Silent.ogg', -- Same sound as UH2; jumping onto ledge
	[CHAR_SOUND_HAHA] = 'Laugh.ogg', -- Landing triple jump
	[CHAR_SOUND_YAHOO] = 'LongJump.ogg', -- Long jump
	[CHAR_SOUND_DOH] = 'Damage.ogg', -- Long jump wall bonk
	[CHAR_SOUND_WHOA] = 'ClimbLedge.ogg', -- Grabbing ledge
	[CHAR_SOUND_EEUH] = 'Silent.ogg', -- Climbing over ledge
	[CHAR_SOUND_WAAAOOOW] = 'Falling.ogg', -- Falling a long distance
	[CHAR_SOUND_TWIRL_BOUNCE] = 'Laugh.ogg', -- Bouncing off of a flower spring
	[CHAR_SOUND_GROUND_POUND_WAH] = 'Silent.ogg', 
	[CHAR_SOUND_HRMM] = 'Snore.ogg', -- Lifting something
	[CHAR_SOUND_HERE_WE_GO] = 'GetStar.ogg', -- Star get
	[CHAR_SOUND_SO_LONGA_BOWSER] = 'ThrowBowser.ogg', -- Throwing Bowser
--DAMAGE
	[CHAR_SOUND_ATTACKED] = 'Damage.ogg', -- Damaged
	[CHAR_SOUND_PANTING] = 'Panting.ogg', -- Low health
	[CHAR_SOUND_ON_FIRE] = 'Falling.ogg', -- Burned
--SLEEP SOUNDS
	[CHAR_SOUND_IMA_TIRED] = 'Silent.ogg', -- Mario feeling tired
	[CHAR_SOUND_YAWNING] = 'Tired.ogg', -- Mario yawning before he sits down to sleep
	[CHAR_SOUND_SNORING1] = 'Snore.ogg', -- Snore Inhale
	[CHAR_SOUND_SNORING2] = 'Snore2.ogg', -- Exhale
	[CHAR_SOUND_SNORING3] = 'Silent.ogg', -- Sleep talking / mumbling
--COUGHING (USED IN THE GAS MAZE)
	[CHAR_SOUND_COUGHING1] = 'Silent.ogg', -- Cough take 1
	[CHAR_SOUND_COUGHING2] = 'Silent.ogg', -- Cough take 2
	[CHAR_SOUND_COUGHING3] = 'Silent.ogg', -- Cough take 3
--DEATH
	[CHAR_SOUND_DYING] = 'Dying.ogg', -- Dying from damage
	[CHAR_SOUND_DROWNING] = 'Drowning.ogg', -- Running out of air underwater
	[CHAR_SOUND_MAMA_MIA] = 'Damage.ogg' -- Booted out of level
}

-- All Located in "actors"
local CAPTABLE_CHAR = {
    normal = smlua_model_util_get_id("drybowser_vanish_geo"),
    wing = smlua_model_util_get_id("drybowser_wing_geo"),
    metal = smlua_model_util_get_id("drybowser_metal_geo"),
}

local PALETTE_CHAR = {
    [PANTS]  = "ffffff",
    [SHIRT]  = "FF001C",
    [GLOVES] = "3C3C3C",
    [SHOES]  = "ffffff",
    [HAIR]   = "FF001C",
    [SKIN]   = "3C3C3C",
    [CAP]    = "3C3C3C",
	[EMBLEM] = "FF001C",
}

--[[
-- All Located in "textures"
local healthMeter = {
    label = {
        left = get_texture_info("template-hp-back-left"),
        right = get_texture_info("template-hp-back-right"),
    },
    pie = {
        [1] = get_texture_info("template-hp-pie-1"),
        [2] = get_texture_info("template-hp-pie-2"),
        [3] = get_texture_info("template-hp-pie-3"),
        [4] = get_texture_info("template-hp-pie-4"),
        [5] = get_texture_info("template-hp-pie-5"),
        [6] = get_texture_info("template-hp-pie-6"),
        [7] = get_texture_info("template-hp-pie-7"),
        [8] = get_texture_info("template-hp-pie-8"),
    }
}
]]

--[[
    Note: If there are some functionalities you don't care
    to add such as palettes, you can freely remove the function
    for it, the only function you require is character_add
]]

---- BOWSER MOVESET:

if _G.bowsMoveset then
    -- Retrieves the custom shell model for this character. Check template_shell.blend for how to set up a shell model properly.
    local E_MODEL_JR_SHELL = smlua_model_util_get_id("drybowsershell_geo")

    -- This is a bitfield of flags that decide which parts of the Bowser Moveset this character will have.
    -- Insert each flag you want to be set, separated by '|' symbols.
    local BOWS_FLAGS_JR =
        _G.bowsMoveset.FLAG_CAN_USE_FIREBALL | _G.bowsMoveset.FLAG_CAN_USE_SHELL | _G.bowsMoveset.FLAG_HEAVY_STEPS | _G.bowsMoveset.FLAG_NO_CAPLESS
        

    -- Flag Options: (sorry the ids are so long pshgkjdf)

    -- _G.bowsMoveset.FLAG_CAN_USE_SHELL
    --- Allows the character to use the shell slide ability.

    -- _G.bowsMoveset.FLAG_CAN_USE_FIREBALL
    --- Allows the character to use the fire breath ability.

    -- _G.bowsMoveset.FLAG_STYLE_ANIMS
    --- Enables Bowser-unique animations for the character.

    -- _G.bowsMoveset.FLAG_SIZE_ANIMS
    --- Adjusts the character's pose in certain animations to suit bowser's large size. (i.e. ledge grab)

    -- _G.bowsMoveset.FLAG_LARGE_HITBOX
    --- Gives the character a larger hitbox size. (37 -> 85 units radius)
    --- This does not affect the hitbox size for level collision, only collisions with objects.

    -- _G.bowsMoveset.FLAG_NO_CAPLESS
    --- The character does not visually lose their 'cap'. Will also use their 'capless' head state in some animations.
    --- Used by Bowser to allow his jaw to open when breathing fire, etc.

    -- _G.bowsMoveset.FLAG_HEAVY_STEPS
    --- Enables heavier landing sound effects for the character.

    -- _G.bowsMoveset.FLAG_ATTACKS
    --- Enables the alternate sliding punch.

    -- _G.bowsMoveset.FLAG_ALL
    --- Shorthand to set all flags.


    -- This function sets up your custom shell model. Feel free to remove this if you aren't using a shell model.
    -- parameters: [your character model], [your shell model], [custom shell sound]
    _G.bowsMoveset.character_add_shell_model(E_MODEL_CUSTOM_MODEL, E_MODEL_JR_SHELL, 'shell_speen.ogg')
    -- This function sets up the flags for your character.
    -- parameters: [your character model], [your flag bitfield]
    _G.bowsMoveset.character_set_bows_flags(E_MODEL_CUSTOM_MODEL, BOWS_FLAGS_JR)
end

local CSloaded = false
local function on_character_select_load()
    CT_CHAR = _G.charSelect.character_add("Dry Bowser", {"Bowser when haves no water", "Bowser Moveset Support! Credit to: Wibblus"}, "Garrulous64", {r = 255, g = 200, b = 200}, E_MODEL_CUSTOM_MODEL, CT_MARIO, TEX_CUSTOM_LIFE_ICON)
    _G.charSelect.character_add_caps(E_MODEL_CUSTOM_MODEL, CAPTABLE_CHAR)
    _G.charSelect.character_add_voice(E_MODEL_CUSTOM_MODEL, VOICETABLE_DEDEDE)
    _G.charSelect.character_add_celebration_star(E_MODEL_CUSTOM_MODEL, E_MODEL_CUSTOM_STAR, TEX_CUSTOM_STAR_ICON)
    _G.charSelect.character_add_palette_preset(E_MODEL_CUSTOM_MODEL, PALETTE_CHAR)
    --_G.charSelect.character_add_health_meter(CT_CHAR, healthMeter)

    CSloaded = true
end

local function on_character_sound(m, sound)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_DEDEDE then return _G.charSelect.voice.sound(m, sound) end
end

local function on_character_snore(m)
    if not CSloaded then return end
    if _G.charSelect.character_get_voice(m) == VOICETABLE_DEDEDE then return _G.charSelect.voice.snore(m) end
end

hook_event(HOOK_ON_MODS_LOADED, on_character_select_load)
hook_event(HOOK_CHARACTER_SOUND, on_character_sound)
hook_event(HOOK_MARIO_UPDATE, on_character_snore)